@@ -10,7 +10,8 @@ from octue import exceptions
 from octue.cloud.storage import GoogleCloudStorageClient
 from octue.mixins import MixinBase, Pathable
 from octue.resources.datafile import TEMPORARY_LOCAL_FILE_CACHE, Datafile
-from octue.resources.tag import TagSet
+from octue.resources.label import LabelSet
+from octue.resources.tag import TagDict
 from tests import TEST_BUCKET_NAME, TEST_PROJECT_NAME
 from ..base import BaseTestCase
 
@@ -29,7 +30,7 @@ class DatafileTestCase(BaseTestCase):
         TEMPORARY_LOCAL_FILE_CACHE.clear()
 
     def create_valid_datafile(self):
-        return Datafile(timestamp=None, path_from=self.path_from, path=self.path, skip_checks=False)
+        return Datafile(path_from=self.path_from, path=self.path, skip_checks=False)
 
     def create_datafile_in_cloud(
         self,
@@ -50,14 +51,13 @@ class DatafileTestCase(BaseTestCase):
         with tempfile.NamedTemporaryFile("w", delete=False) as temporary_file:
             temporary_file.write(contents)
 
-        timestamp = kwargs.pop("timestamp", None)
-        datafile = Datafile(path=temporary_file.name, timestamp=timestamp, **kwargs)
+        datafile = Datafile(path=temporary_file.name, **kwargs)
         datafile.to_cloud(project_name=project_name, bucket_name=bucket_name, path_in_bucket=path_in_bucket)
         return datafile, project_name, bucket_name, path_in_bucket, contents
 
     def test_instantiates(self):
         """Ensures a Datafile instantiates using only a path and generates a uuid ID"""
-        df = Datafile(timestamp=None, path="a_path")
+        df = Datafile(path="a_path")
         self.assertTrue(isinstance(df.id, str))
         self.assertEqual(type(uuid.UUID(df.id)), uuid.UUID)
         self.assertIsNone(df.sequence)
@@ -82,35 +82,35 @@ class DatafileTestCase(BaseTestCase):
 
     def test_gt(self):
         """Test that datafiles can be ordered using the greater-than operator."""
-        a = Datafile(timestamp=None, path="a_path")
-        b = Datafile(timestamp=None, path="b_path")
+        a = Datafile(path="a_path")
+        b = Datafile(path="b_path")
         self.assertTrue(a < b)
 
     def test_gt_with_wrong_type(self):
         """Test that datafiles cannot be ordered compared to other types."""
         with self.assertRaises(TypeError):
-            Datafile(timestamp=None, path="a_path") < "hello"
+            Datafile(path="a_path") < "hello"
 
     def test_lt(self):
         """Test that datafiles can be ordered using the less-than operator."""
-        a = Datafile(timestamp=None, path="a_path")
-        b = Datafile(timestamp=None, path="b_path")
+        a = Datafile(path="a_path")
+        b = Datafile(path="b_path")
         self.assertTrue(b > a)
 
     def test_lt_with_wrong_type(self):
         """Test that datafiles cannot be ordered compared to other types."""
         with self.assertRaises(TypeError):
-            Datafile(timestamp=None, path="a_path") > "hello"
+            Datafile(path="a_path") > "hello"
 
     def test_checks_fail_when_file_doesnt_exist(self):
         path = "not_a_real_file.csv"
         with self.assertRaises(exceptions.FileNotFoundException) as error:
-            Datafile(timestamp=None, path=path, skip_checks=False)
+            Datafile(path=path, skip_checks=False)
         self.assertIn("No file found at", error.exception.args[0])
 
     def test_conflicting_extension_fails_check(self):
         with self.assertRaises(exceptions.InvalidInputException) as error:
-            Datafile(timestamp=None, path_from=self.path_from, path=self.path, skip_checks=False, extension="notcsv")
+            Datafile(path_from=self.path_from, path=self.path, skip_checks=False, extension="notcsv")
 
         self.assertIn("Extension provided (notcsv) does not match file extension", error.exception.args[0])
 
@@ -151,6 +151,7 @@ class DatafileTestCase(BaseTestCase):
             "timestamp",
             "sequence",
             "tags",
+            "labels",
             "_cloud_metadata",
         }
 
@@ -171,7 +172,7 @@ class DatafileTestCase(BaseTestCase):
     def test_is_in_cloud(self):
         """Test whether a file is in the cloud or not can be determined."""
         self.assertFalse(self.create_valid_datafile().is_in_cloud)
-        self.assertTrue(Datafile(timestamp=None, path="gs://hello/file.txt").is_in_cloud)
+        self.assertTrue(Datafile(path="gs://hello/file.txt").is_in_cloud)
 
     def test_from_cloud_with_bare_file(self):
         """Test that a Datafile can be constructed from a file on Google Cloud storage with no custom metadata."""
@@ -190,7 +191,8 @@ class DatafileTestCase(BaseTestCase):
         self.assertEqual(datafile.path, f"gs://{TEST_BUCKET_NAME}/{path_in_bucket}")
         self.assertEqual(datafile.cluster, 0)
         self.assertEqual(datafile.sequence, None)
-        self.assertEqual(datafile.tags, TagSet())
+        self.assertEqual(datafile.tags, TagDict())
+        self.assertEqual(datafile.labels, LabelSet())
         self.assertTrue(isinstance(datafile.size_bytes, int))
         self.assertTrue(isinstance(datafile._last_modified, float))
         self.assertTrue(isinstance(datafile.hash_value, str))
@@ -201,18 +203,20 @@ class DatafileTestCase(BaseTestCase):
             timestamp=datetime.now(tz=timezone.utc),
             cluster=0,
             sequence=1,
-            tags={"blah:shah:nah", "blib", "glib"},
+            labels={"blah-shah-nah", "blib", "glib"},
+            tags={"good": True, "how_good": "very"},
         )
+        gs_path = f"gs://{TEST_BUCKET_NAME}/{path_in_bucket}"
+        downloaded_datafile = Datafile.from_cloud(project_name, cloud_path=gs_path)
 
-        downloaded_datafile = Datafile.from_cloud(project_name, bucket_name, path_in_bucket)
-
-        self.assertEqual(downloaded_datafile.path, f"gs://{TEST_BUCKET_NAME}/{path_in_bucket}")
+        self.assertEqual(downloaded_datafile.path, gs_path)
         self.assertEqual(downloaded_datafile.id, datafile.id)
         self.assertEqual(downloaded_datafile.timestamp, datafile.timestamp)
         self.assertEqual(downloaded_datafile.hash_value, datafile.hash_value)
         self.assertEqual(downloaded_datafile.cluster, datafile.cluster)
         self.assertEqual(downloaded_datafile.sequence, datafile.sequence)
         self.assertEqual(downloaded_datafile.tags, datafile.tags)
+        self.assertEqual(downloaded_datafile.labels, datafile.labels)
         self.assertEqual(downloaded_datafile.size_bytes, datafile.size_bytes)
         self.assertTrue(isinstance(downloaded_datafile._last_modified, float))
 
@@ -257,9 +261,11 @@ class DatafileTestCase(BaseTestCase):
         datafile, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud(cluster=0)
 
         datafile.cluster = 3
-        datafile.to_cloud(project_name, bucket_name, path_in_bucket)
+        datafile.to_cloud(project_name, bucket_name=bucket_name, path_in_bucket=path_in_bucket)
 
-        self.assertEqual(Datafile.from_cloud(project_name, bucket_name, path_in_bucket).cluster, 3)
+        self.assertEqual(
+            Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket).cluster, 3
+        )
 
     def test_to_cloud_does_not_update_cloud_metadata_if_update_cloud_metadata_is_false(self):
         """Test that calling Datafile.to_cloud with `update_cloud_metadata=False` doesn't update the cloud metadata."""
@@ -267,16 +273,20 @@ class DatafileTestCase(BaseTestCase):
         datafile.cluster = 3
 
         with patch("octue.resources.datafile.Datafile.update_cloud_metadata") as mock:
-            datafile.to_cloud(project_name, bucket_name, path_in_bucket, update_cloud_metadata=False)
+            datafile.to_cloud(
+                project_name, bucket_name=bucket_name, path_in_bucket=path_in_bucket, update_cloud_metadata=False
+            )
             self.assertFalse(mock.called)
 
-        self.assertEqual(Datafile.from_cloud(project_name, bucket_name, path_in_bucket).cluster, 0)
+        self.assertEqual(
+            Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket).cluster, 0
+        )
 
     def test_to_cloud_does_not_update_metadata_if_no_metadata_change_has_been_made(self):
         """Test that Datafile.to_cloud does not try to update cloud metadata if no metadata change has been made."""
         _, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud(cluster=0)
 
-        datafile = Datafile.from_cloud(project_name, bucket_name, path_in_bucket)
+        datafile = Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket)
 
         with patch("octue.resources.datafile.Datafile.update_cloud_metadata") as mock:
             datafile.to_cloud()
@@ -296,7 +306,7 @@ class DatafileTestCase(BaseTestCase):
         provided.
         """
         _, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud()
-        datafile = Datafile.from_cloud(project_name, bucket_name, path_in_bucket)
+        datafile = Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket)
         datafile.to_cloud()
 
     def test_to_cloud_does_not_try_to_update_file_if_no_change_has_been_made_locally(self):
@@ -311,10 +321,12 @@ class DatafileTestCase(BaseTestCase):
         """Test that a cloud datafile's metadata can be updated."""
         _, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud()
 
-        new_datafile = Datafile(path="glib.txt", timestamp=None, cluster=32)
-        new_datafile.update_cloud_metadata(project_name, bucket_name, path_in_bucket)
+        new_datafile = Datafile(path="glib.txt", cluster=32)
+        new_datafile.update_cloud_metadata(project_name, bucket_name=bucket_name, path_in_bucket=path_in_bucket)
 
-        self.assertEqual(Datafile.from_cloud(project_name, bucket_name, path_in_bucket).cluster, 32)
+        self.assertEqual(
+            Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket).cluster, 32
+        )
 
     def test_update_cloud_metadata_works_with_implicit_cloud_location_if_cloud_location_previously_provided(self):
         """Test that datafile.update_metadata works with an implicit cloud location if the cloud location has been
@@ -322,11 +334,13 @@ class DatafileTestCase(BaseTestCase):
         """
         _, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud()
 
-        datafile = Datafile.from_cloud(project_name, bucket_name, path_in_bucket)
+        datafile = Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket)
         datafile.cluster = 32
         datafile.update_cloud_metadata()
 
-        self.assertEqual(Datafile.from_cloud(project_name, bucket_name, path_in_bucket).cluster, 32)
+        self.assertEqual(
+            Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket).cluster, 32
+        )
 
     def test_update_cloud_metadata_raises_error_if_no_cloud_location_provided_and_datafile_not_from_cloud(self):
         """Test that trying to update a cloud datafile's metadata with no cloud location provided when the datafile was
@@ -340,7 +354,7 @@ class DatafileTestCase(BaseTestCase):
     def test_get_local_path(self):
         """Test that a file in the cloud can be temporarily downloaded and its local path returned."""
         _, project_name, bucket_name, path_in_bucket, contents = self.create_datafile_in_cloud()
-        datafile = Datafile.from_cloud(project_name, bucket_name, path_in_bucket)
+        datafile = Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket)
 
         with open(datafile.get_local_path()) as f:
             self.assertEqual(f.read(), contents)
@@ -348,7 +362,7 @@ class DatafileTestCase(BaseTestCase):
     def test_get_local_path_with_cached_file_avoids_downloading_again(self):
         """Test that attempting to download a cached file avoids downloading it again."""
         _, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud()
-        datafile = Datafile.from_cloud(project_name, bucket_name, path_in_bucket)
+        datafile = Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket)
 
         # Download for first time.
         datafile.get_local_path()
@@ -365,7 +379,7 @@ class DatafileTestCase(BaseTestCase):
         with tempfile.NamedTemporaryFile("w", delete=False) as temporary_file:
             temporary_file.write(file_contents)
 
-        datafile = Datafile(timestamp=None, path=temporary_file.name)
+        datafile = Datafile(path=temporary_file.name)
 
         with datafile.open() as f:
             self.assertEqual(f.read(), file_contents)
@@ -377,7 +391,7 @@ class DatafileTestCase(BaseTestCase):
         with tempfile.NamedTemporaryFile("w", delete=False) as temporary_file:
             temporary_file.write(file_contents)
 
-        datafile = Datafile(timestamp=None, path=temporary_file.name)
+        datafile = Datafile(path=temporary_file.name)
 
         with datafile.open("w") as f:
             f.write("hello")
@@ -388,7 +402,7 @@ class DatafileTestCase(BaseTestCase):
     def test_open_with_reading_cloud_file(self):
         """Test that a cloud datafile can be opened for reading."""
         _, project_name, bucket_name, path_in_bucket, contents = self.create_datafile_in_cloud()
-        datafile = Datafile.from_cloud(project_name, bucket_name, path_in_bucket)
+        datafile = Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket)
 
         with datafile.open() as f:
             self.assertEqual(f.read(), contents)
@@ -396,7 +410,7 @@ class DatafileTestCase(BaseTestCase):
     def test_open_with_writing_to_cloud_file(self):
         """Test that a cloud datafile can be opened for writing and that both the remote and local copies are updated."""
         _, project_name, bucket_name, path_in_bucket, original_contents = self.create_datafile_in_cloud()
-        datafile = Datafile.from_cloud(project_name, bucket_name, path_in_bucket)
+        datafile = Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket)
 
         new_file_contents = "nanana"
 
@@ -486,7 +500,7 @@ class DatafileTestCase(BaseTestCase):
         temporary_file = tempfile.NamedTemporaryFile("w", delete=False)
         contents = "Here is the content."
 
-        with Datafile(path=temporary_file.name, timestamp=None, mode="w") as (datafile, f):
+        with Datafile(path=temporary_file.name, mode="w") as (datafile, f):
             f.write(contents)
 
         # Check that the cloud file has been updated.
@@ -499,13 +513,18 @@ class DatafileTestCase(BaseTestCase):
         new_contents = "Here is the new content."
         self.assertNotEqual(original_content, new_contents)
 
-        with Datafile.from_cloud(project_name, bucket_name, path_in_bucket, mode="w") as (datafile, f):
-            datafile.add_tags("blue")
+        with Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket, mode="w") as (
+            datafile,
+            f,
+        ):
+            datafile.add_labels("blue")
             f.write(new_contents)
 
         # Check that the cloud metadata has been updated.
-        re_downloaded_datafile = Datafile.from_cloud(project_name, bucket_name, path_in_bucket)
-        self.assertTrue("blue" in re_downloaded_datafile.tags)
+        re_downloaded_datafile = Datafile.from_cloud(
+            project_name, bucket_name=bucket_name, datafile_path=path_in_bucket
+        )
+        self.assertTrue("blue" in re_downloaded_datafile.labels)
 
         # The file cache must be cleared so the modified cloud file is downloaded.
         re_downloaded_datafile.clear_from_file_cache()
@@ -513,3 +532,25 @@ class DatafileTestCase(BaseTestCase):
         # Check that the cloud file has been updated.
         with re_downloaded_datafile.open() as f:
             self.assertEqual(f.read(), new_contents)
+
+    def test_metadata(self):
+        """Test that the metadata method namespaces the metadata names when required."""
+        datafile = self.create_valid_datafile()
+
+        self.assertEqual(
+            datafile.metadata().keys(),
+            {
+                "octue__id",
+                "octue__timestamp",
+                "octue__cluster",
+                "octue__sequence",
+                "octue__tags",
+                "octue__labels",
+                "octue__sdk_version",
+            },
+        )
+
+        self.assertEqual(
+            datafile.metadata(use_octue_namespace=False).keys(),
+            {"id", "timestamp", "cluster", "sequence", "tags", "labels", "sdk_version"},
+        )
